"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.liftListById = exports.liftListBy = exports.mergeByKeys = exports.flatMerge = exports.flatCombine = exports.Model = exports.L = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _rx = require("rx");

var _rx2 = _interopRequireDefault(_rx);

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _partial = require("partial.lenses");

var _partial2 = _interopRequireDefault(_partial);

var _model = require("./model");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _keys = function _keys(x) {
  return x ? Object.keys(x) : [];
};
var extend = Object.assign;
var isFun = function isFun(x) {
  return x && typeof x === "function";
};

var flattenBy = function flattenBy(list$$, keys, fn) {
  return keys.reduce(function (acc, k) {
    return _extends({}, acc, _defineProperty({}, k, list$$.flatMapLatest(fn(k))));
  }, {});
};

// ==== public stuff ====

var L = exports.L = _partial2.default;

var Model = exports.Model = _model.makeModelDriver;

var flatCombine = exports.flatCombine = function flatCombine(list$$) {
  for (var _len = arguments.length, keys = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    keys[_key - 1] = arguments[_key];
  }

  return flattenBy(list$$, keys, function (k) {
    return function (xs) {
      return !xs.length ? _rx.Observable.just([]) : _rx.Observable.combineLatest.apply(_rx.Observable, _toConsumableArray(xs.map(function (x) {
        return x[k];
      })));
    };
  });
};

var flatMerge = exports.flatMerge = function flatMerge(list$$) {
  for (var _len2 = arguments.length, keys = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    keys[_key2 - 1] = arguments[_key2];
  }

  return flattenBy(list$$, keys, function (k) {
    return function (xs) {
      return _rx.Observable.merge(xs.map(function (x) {
        return x[k] || _rx.Observable.empty();
      }));
    };
  });
};

var mergeByKeys = exports.mergeByKeys = function mergeByKeys() {
  for (var _len3 = arguments.length, objects = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    objects[_key3] = arguments[_key3];
  }

  var merged = {};
  objects.forEach(function (o) {
    return _keys(o).forEach(function (k) {
      return merged[k] = merged[k] ? merged[k].merge(o[k]) : o[k];
    });
  });
  return merged;
};

var liftListBy = exports.liftListBy = function () {
  var liftListBy = _ramda2.default.curryN(3, function liftListBy(identity, list$, it) {
    var replay = arguments.length <= 3 || arguments[3] === undefined ? ["DOM"] : arguments[3];

    return _rx.Observable.using(function () {
      return new Cache(replay);
    }, function (cache) {
      return list$.distinctUntilChanged(function (items) {
        return items.map(function (item) {
          return identity(item);
        });
      }, function (a, b) {
        if (a.length !== b.length) return false;
        for (var i = 0; i < a.length; i++) {
          if (a[i] !== b[i]) return false;
        }
        return true;
      }).map(function (items) {
        var itemByKey = {};
        items.forEach(function (item, idx) {
          return itemByKey[identity(item)] = { item: item, idx: idx };
        });
        items.forEach(function (item, idx) {
          var key = identity(item);
          if (!cache.contains(key)) {
            var item$ = isFun(list$.lens) ? list$.lens(L.find(_ramda2.default.compose(_ramda2.default.equals(key), identity))) : null;
            cache.put(key, it(key, item$), idx);
          } else {
            cache.reIndex(key, idx);
          }
        });
        cache.keys().forEach(function (key) {
          if (!(key in itemByKey)) {
            cache.del(key);
          }
        });
        return cache.list();
      });
    }).shareReplay(1);
  });

  function Cache(toReplay) {
    this.cache = {};
    this.toReplay = toReplay.reduce(function (acc, k) {
      return _extends({}, acc, _defineProperty({}, k, true));
    }, {});
  }

  extend(Cache.prototype, {
    contains: function contains(key) {
      return key in this.cache;
    },
    put: function put(key, sinks, idx) {
      var _this = this;

      var streams = {};
      var disposable = new _rx2.default.CompositeDisposable();
      _keys(sinks).map(function (name) {
        var sink$ = _this.toReplay[name] ? sinks[name].replay(null, 1) : sinks[name].publish();
        disposable.add(sink$.connect());
        streams[name] = sink$;
      });
      this.cache[key] = {
        key: key,
        streams: streams,
        disposable: disposable,
        idx: idx
      };
    },
    reIndex: function reIndex(key, idx) {
      this.cache[key].idx = idx;
    },
    del: function del(key) {
      var cached = this.cache[key];
      if (cached) {
        delete this.cache[key];
        cached.disposable.dispose();
      }
    },
    keys: function keys() {
      return _keys(this.cache);
    },
    list: function list() {
      var _this2 = this;

      var list = _keys(this.cache).map(function (k) {
        return _this2.cache[k];
      });
      list.sort(function (a, b) {
        return a.idx - b.idx;
      });
      return list.map(function (x) {
        return x.streams;
      });
    },
    dispose: function dispose() {
      var _this3 = this;

      this.toReplay = [];
      _keys(this.cache).forEach(function (key) {
        _this3.del(key);
      });
    }
  });

  return liftListBy;
}();

var liftListById = exports.liftListById = liftListBy(_ramda2.default.prop("id"));