"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeModelDriver = makeModelDriver;

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _partial = require("partial.lenses");

var _partial2 = _interopRequireDefault(_partial);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function makeModelDriver(initial) {
  var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var ID = {};
  var _opts$logging = // eslint-disable-line
  opts.logging;
  var logging = _opts$logging === undefined ? false : _opts$logging;
  var _opts$info = opts.info;
  var info = _opts$info === undefined ? function () {
    var _console;

    return (_console = console).info.apply(_console, arguments);
  } : _opts$info;
  var _opts$warn = opts.warn;
  var // eslint-disable-line
  warn = _opts$warn === undefined ? function () {
    var _console2;

    return (_console2 = console).warn.apply(_console2, arguments);
  } : _opts$warn;


  return function ModelDriver(mod$) {
    var state$ = mod$.filter(function (m) {
      return m && m.ID === ID || warn("Received modification that was not created by using 'M.mods'. Ignoring..") && false;
    }).startWith(initial).scan(function (s, _ref) {
      var mod = _ref.mod;
      return mod(s);
    });

    if (logging) {
      state$ = state$.do(function (s) {
        return info("New state:", s);
      });
    }
    return model(state$, _ramda2.default.lens(_ramda2.default.identity, _ramda2.default.nthArg(0)));

    function model(state$, stateLens) {
      var val$ = state$.distinctUntilChanged().shareReplay(1);

      return Object.assign(val$, {
        lens: function lens(l) {
          for (var _len = arguments.length, ls = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            ls[_key - 1] = arguments[_key];
          }

          return model(val$.map(_partial2.default.view(_partial2.default.apply(undefined, [l].concat(ls)))), _partial2.default.apply(undefined, [stateLens, l].concat(ls)));
        },
        mod: function mod(mod$) {
          return mod$.map(function (mod) {
            return { mod: _ramda2.default.over(stateLens, mod), ID: ID };
          });
        },
        set: function set(val$) {
          return val$.map(function (val) {
            return { mod: _ramda2.default.over(stateLens, _ramda2.default.always(val)), ID: ID };
          });
        }
      });
    }
  };
}